import { continuedIndent, indentNodeProp, foldNodeProp, LRLanguage, } from "@codemirror/language";
import { styleTags, tags } from "@codemirror/highlight";
import { parser } from "lezer-taslx";
const styleNodeProp = styleTags({
    "namespace literal map": tags.keyword,
    Comment: tags.comment,
    "NamespaceDefinition/Prefix": tags.name,
    "NamespaceDefinition/NamespaceURI": [tags.url, tags.link],
    "LiteralDefinition/Name": tags.typeName,
    "MapDeclaration/Source MapDeclaration/Target": tags.className,
    Identifier: tags.variableName,
    URI: tags.url,
    "Literal/String": tags.string,
    "Literal/Tag": tags.separator,
    "Literal/Identifier": tags.typeName,
    "OpenBrace CloseBrace": tags.brace,
    "OpenBracket CloseBracket": tags.bracket,
    "OpenParen CloseParen": tags.paren,
    "Slot/Key Case/Key": tags.propertyName,
    "Pointer Projector Injector": tags.operator,
    "Dereference/Key": tags.className,
    "Projection/Key Injection/Key": tags.name,
    "RightMap LeftMap": tags.separator,
});
export const taslxLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Construction: continuedIndent({ except: /^\s*\}/ }),
                Match: continuedIndent({ except: /^\s*\]/ }),
            }),
            foldNodeProp.add({
                Construction(subtree) {
                    return { from: subtree.from + 1, to: subtree.to - 1 };
                },
                Match(subtree) {
                    return { from: subtree.from + 1, to: subtree.to - 1 };
                },
            }),
            styleNodeProp,
        ],
    }),
    languageData: {
        closeBrackets: { brackets: ["[", "{", "("] },
        indentOnInput: /^\s*[\}\]]$/,
        commentTokens: { line: "#" },
    },
});
